C***********************************************************************
C* PROGRAM L3B42main0ss                                                *
C*                                                                     *
C* This program is the main driver program for SSMIS GPROF2004V.  It   *
C* reads the input file names from a list file and calls RETRIEVAL for *
C* each one, to run GPROF2004V and produce global 0.25 x 0.25-degree   *
C* forward-gridded output files.                                       *
C*                                                                     *
C* L3B42main0ss cinfile                                                *
C*                                                                     *
C* cinfile is a simple ASCII file that contains the command-line       *
C* arguments for L3B42main0ss.  cinfile contains the following         *
C* arguments, one per line:                                            *
C*                                                                     *
C* Input command-line arguments:                                       *
C*   cjobid      CHAR    PPS job ID                                    *
C*   satid       CHAR    satellite ID (F16, F17, F18, F19)             *
C*   infile      CHAR    names of CLASS TDR (antenna temp.) files,     *
C*                       array of dimension (10000)                    *
C*   atime0      CHAR    beginning of time window to process           *
C*                       (of the form YYYYMMDD"T"HH:MM:SS, w/o quotes) *
C*   atime1      CHAR    end of time window to process                 *
C*                       (of the form YYYYMMDD"T"HH:MM:SS, w/o quotes) *
C*   granule_list_file                                                 *
C*               CHAR    dynamically-built file containing start/end   *
C*                      times, output filenames of all orbits processed*
C*   outpath_g   CHAR    Path (directory) for gridded output, including*
C*                       trailing slash (/)                            *
C*   debug_flag  CHAR    Flag for dumping debug information            *
C*                                                                     *
C*   In the V7 code, timefile, a dynamically-built file containing     *
C*   start/end times of recently-processed input files, becomes a local*
C*   file that incorporates cjobid in its title.                       *
C*                                                                     *
C**                                                                    *
C* Log:                                                                *
C* E.Nelkin/SSAI        11/11                                          *
C* E.Nelkin/SSAI        11/11   Read auxiliary coefficient and LUT     *
C*                              files here and pass relevant arrays to *
C*                              subroutines "extract" and "ingest"     *
C* E.Nelkin/SSAI        11/11   Pass sat_name into subroutine retrieval*
C* E.Nelkin/SSAI         9/12   Non-PPS version for Bob Joyce; delete  *
C*                              granule_list_file                      *
C***********************************************************************
	IMPLICIT NONE
CCC
        INCLUDE 'parameters.inc'
C
        INTEGER*4       numfiles
c	CHARACTER*96   infile(10000)
c       CHARACTER*93   infile(10000), linfile
        CHARACTER*65   infile(10000), linfile
	CHARACTER*256   cinfile, timefile, outpath_g,
     &                  cjobid, satid, output_file, indecodefl
        CHARACTER*256   coeff_file
        CHARACTER*17    atime0, atime1
        CHARACTER*3     sat_name
        INTEGER*4       itime0(6), itime1(6), nsec0, nsec1
        INTEGER*4       i, j, k, l_cjobid, l_if, l_gf, l_sid, l_tf,
     &                  l_cfe, l_cfi, l_lutL, l_lutO, iret
        INTEGER*4       ione, izero, iunit_t, iunit_c, iunit_s
        CHARACTER*1     debug_flag
        LOGICAL         in_use, do_debug
C
        INTEGER*4       beam
        CHARACTER*60    trash
        REAL*4          coeff_19v(NPIX/2)      !array 19V data in Kelvin
        REAL*4          coeff_19h(NPIX/2)      !array 19H data in Kelvin
        REAL*4          coeff_22v(NPIX/2)      !array 22V data in Kelvin
        REAL*4          coeff_37v(NPIX/2)      !array 37V data in Kelvin
        REAL*4          coeff_37h(NPIX/2)      !array 37H data in Kelvin

        REAL*4          coeff_91v(NPIX)        !array 91V data in Kelvin
        REAL*4          coeff_91h(NPIX)        !array 91h data in Kelvin
        REAL*4          coeff_189(NPIX)        !array 189 data in Kelvin
        REAL*4          coeff_186(NPIX)        !array 186 data in Kelvin
        REAL*4          coeff_184(NPIX)        !array 184 data in Kelvin
        REAL*4          coeff_150(NPIX)        !array 150 data in Kelvin
C
        !=== lut variables (histogram matching SSMI-SSMIS) ===
        REAL*4          add_oce(161,7), add_lnd(161,7)
        REAL*4          SSMIS_OCE(161), SSMIS_LND(161)
        character*3 month(12), mnth
        integer*4 imth, idy, ihr, iyr, iyrmndy

        character*8 sid
        integer id
 
       data month/"Jan","Feb","Mar","Apr","May","Jun",
     1           "Jul","Aug","Sep","Oct","Nov","Dec"/
 
C
C***********************************************************************
C       Read the command-line arguments from input stream and return
C       them to L3B42main0ss for further processing.
C***********************************************************************
C
       call getarg(2, sid)
       read(sid, *) id

c      indecodefl='/cpc/prod_tst/cpcsat/cmorph/data/nesdis/ssmi/gprof7/d
c    &mspID/decodedatefile'
c      write(indecodefl(57:58), '(i2.2)') id
c      open (unit=28, file = indecodefl)
c      write(*,*) 'decode file date: ', indecodefl
c      read (28,'(4x,a3,i3,i3,11x,i4)') mnth, idy, ihr, iyr
c      close (28)
c      do 117 imth=1,12
c117    if (mnth.eq.month(imth)) goto 118
c118   continue
c      iyrmndy = iyr*10000+imth*100+idy
c      write (6,'(4x,a3,i3,i3,11x,i4,i9)') mnth, idy, ihr, iyr,iyrmndy

        CALL READARGS (cinfile, cjobid, satid, infile, atime0, atime1,
     &                 outpath_g, debug_flag, numfiles)

c       linfile = infile(1)
c       read(linfile(81:82), '(i2.2)') iyr
c       if (iyr.lt.50) then
c        iyr = iyr + 2000
c       elseif (iyr.gt.50) then
c        iyr = iyr + 1900
c       endif
c       write(6,'(/,''input TDR file year '',i4)') iyr
c       write(6,'(/,''input TDR file '',a120)') linfile
c    Create output ascii file using 2nd generation CMORPH standards  
c       output_file = '/cpc/prod_tst/cpcsat/cmorph/data/nesdis/ssmi/gpro
c    &f7/dmspID/TDRRAW1/DMSPID__SSMIS_V-GPROF7___YYYYJDY.SHHMM.EHHHH.B##
c    &#####.txt'
c       write(output_file(57:58),'(i2.2)') id
c       write(output_file(72:73),'(i2.2)') id
c       write (output_file(93:96),'(i4)') iyr
c       output_file(97:99)   = linfile(83:85)
c       output_file(102:105) = linfile(88:91)
c       output_file(108:111) = linfile(94:97)
c       output_file(114:120) = linfile(100:106)
c       write(6,'(/,''output decoded file '',a120)') output_file
c       open(95,file=output_file,STATUS = 'new')

C
        do_debug = .FALSE.
        IF ( debug_flag .EQ. 'Y' .OR. debug_flag .EQ. 'y' .OR.
     &       debug_flag .EQ. 'T' .OR. debug_flag .EQ. 't' )
     &     do_debug = .TRUE.
C
        CALL ST_LSTR ( cjobid, l_cjobid, iret )
 
        IF (iret .NE. 0) THEN
           IF (do_debug) WRITE (*,*)
     &         'ERROR: UNABLE TO DETERMINE LENGTH OF CJOBID ',
     &         cjobid, '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
           STOP
        ENDIF
C                                                             
C       Identify the boundary times within which to process data.
C                                                             
        CALL ST_NUMB ( atime0(1:4),   itime0(1), iret )         
        CALL ST_NUMB ( atime0(5:6),   itime0(2), iret )         
        CALL ST_NUMB ( atime0(7:8),   itime0(3), iret )         
        CALL ST_NUMB ( atime0(10:11), itime0(4), iret )         
        CALL ST_NUMB ( atime0(13:14), itime0(5), iret )         
        CALL ST_NUMB ( atime0(16:17), itime0(6), iret )         
C                                                             
        CALL T2SEC1987(itime0, nsec0, iret)                     
 
C                                                             
        CALL ST_NUMB ( atime1(1:4),   itime1(1), iret )         
        CALL ST_NUMB ( atime1(5:6),   itime1(2), iret )         
        CALL ST_NUMB ( atime1(7:8),   itime1(3), iret )         
        CALL ST_NUMB ( atime1(10:11), itime1(4), iret )         
        CALL ST_NUMB ( atime1(13:14), itime1(5), iret )         
        CALL ST_NUMB ( atime1(16:17), itime1(6), iret )         
C                                                             
        CALL T2SEC1987(itime1, nsec1, iret)
 
C
C       Set up the timefile name and initialize the file.  timefile is
C       a dynamically-built file containing start/end times of recently
C       processed input files.
C
        timefile = cjobid(1:l_cjobid)//'.processed'
        CALL ST_LSTR ( timefile, l_tf, iret )
        IF (iret .NE. 0) THEN
           IF (do_debug) WRITE (*,*)
     &         'ERROR: UNABLE TO DETERMINE LENGTH OF ',
     &         'PROCESSED-TIMES FILE NAME ', timefile, ' ;TERMINATING ',
     &         'EXECUTION; PROGRAM L3B42MAIN0SS'
           STOP
        ENDIF
C
        ione = 1
        izero = 0
        in_use = .TRUE.
        iunit_t  = 2
        DO  WHILE ( in_use )
            iunit_t = iunit_t + 1
            INQUIRE ( iunit_t, OPENED=in_use )
        END DO
        OPEN ( iunit_t, file = timefile(1:l_tf), status = 'new',
     &         err = 20 )
        GO TO 22
   20   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM OPENING PROCESSED-TIMES FILE ',
     &     timefile(1:l_tf),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   22   CONTINUE
        WRITE( iunit_t, 101 ) ione
  101   FORMAT(i2)
        WRITE( iunit_t, 102 ) izero, izero
  102   FORMAT(i9,2x,i9)
C
        CALL ST_LSTR ( satid, l_sid, iret )
        IF  (iret .NE. 0) THEN
            IF (do_debug) WRITE (*,*)
     &                'ERROR: UNABLE TO DETERMINE LENGTH OF SATELLITE ',
     &                'ID ', satid, ' ;TERMINATING ',
     &                'EXECUTION; PROGRAM L3B42MAIN0SS'
            STOP
        END IF
C
        IF ( satid(1:l_sid) .EQ. 'F16' .OR. satid(1:l_sid) .EQ. 'f16' )
     &                                                              THEN
           sat_name = 'F16'
        ELSE IF ( satid(1:l_sid) .EQ. 'F17' .OR. satid(1:l_sid) .EQ.
     &                                                      'f17' ) THEN
           sat_name = 'F17'
        ELSE IF ( satid(1:l_sid) .EQ. 'F18' .OR. satid(1:l_sid) .EQ.
     &                                                      'f18' ) THEN
           sat_name = 'F18'
C
C As of Nov. 2011, F19 and F20 do not yet exist.  Nevertheless, account
C for these satellites, which would necessitate acquiring the
C relevant coefficient files, but would not require modifying this code.
C
        ELSE IF ( satid(1:l_sid) .EQ. 'F19' .OR. satid(1:l_sid) .EQ.
     &                                                      'f19' ) THEN
           sat_name = 'F19'
        ELSE IF ( satid(1:l_sid) .EQ. 'F20' .OR. satid(1:l_sid) .EQ.
     &                                                      'f20' ) THEN
           sat_name = 'F20'
        ELSE
           IF (do_debug) WRITE (*,*)
     &               'ERROR: INVALID ENTRY ', satid(1:l_sid),
     &               ' FOR SATELLITE ID; TERMINATING EXECUTION;',
     &               ' PROGRAM L3B42MAIN0SS'
           STOP
        ENDIF
C
C-----------------------------------------------------------------------
C Rather than reading the same auxiliary files every time a new input
C file is encountered, read them all in once here, and pass the relevant
C arrays to subroutine retrieval and subsequently to the subroutines
C extract-ssmis-tdrr and ingest-ssmis-tdrr as appropriate.
C-----------------------------------------------------------------------
C
C Read the arrays that were formerly read within extract-ssmis-tdrr:
C
        !==== Read bias correction coeff data for SSMIS TDR file ======
        ! ASCII file, pretty fast...

        coeff_file='ssmis_'//sat_name(1:3)//
     &             '_env_scan_correction_coefficients'
        CALL ST_LSTR(coeff_file, l_cfe, iret)
        IF (iret .NE. 0) THEN
           IF (do_debug) WRITE (*,*)
     &         'ERROR: UNABLE TO DETERMINE LENGTH OF SSMIS ENV ',
     &         'COEFFICIENT FILE ', coeff_file,
     &         '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
           STOP
        ENDIF
        in_use = .TRUE.
        iunit_c  = 2
        DO  WHILE ( in_use )
            iunit_c = iunit_c + 1
            INQUIRE ( iunit_c, OPENED=in_use )
        END DO
        OPEN(iunit_c,file=coeff_file(1:l_cfe), status='old', err=24)
        GO TO 26
C
   24   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM OPENING SSMIS ENV COEFFICIENT FILE ',
     &     coeff_file(1:l_cfe),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   26   CONTINUE
C
        DO k=1,93
           IF (k.le.3) THEN
              READ(iunit_c, '(A60)', ERR=28) trash
           ELSE
              READ(iunit_c,'(i4,5(1x,f8.6))', ERR=28) beam,
     &            coeff_19h(k-3), coeff_19v(k-3), coeff_22v(k-3),
     &            coeff_37h(k-3), coeff_37v(k-3)
           ENDIF
        END DO
        GO TO 29
C
   28   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM READING SSMIS ENV COEFFICIENT FILE ',
     &     coeff_file(1:l_cfe),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   29   CONTINUE
        CLOSE(iunit_c)

        coeff_file='ssmis_'//sat_name(1:3)//
     &             '_img_scan_correction_coefficients'
        CALL ST_LSTR(coeff_file, l_cfi, iret)
        IF (iret .NE. 0) THEN
           IF (do_debug) WRITE (*,*)
     &         'ERROR: UNABLE TO DETERMINE LENGTH OF SSMIS IMG ',
     &         'COEFFICIENT FILE ', coeff_file,
     &         '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
           STOP
        ENDIF
        OPEN(iunit_c,file=coeff_file(1:l_cfi), status='old', err=31)
        GO TO 32
C
   31   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM OPENING SSMIS IMG COEFFICIENT FILE ',
     &     coeff_file(1:l_cfi),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   32   CONTINUE
        DO k=1,183
           IF (k.le.3) THEN
               READ(iunit_c, '(A60)', ERR=33) trash
           ELSE
               READ(iunit_c,'(i4,6(1x,f8.6))', ERR=33) beam,
     &             coeff_150(k-3), coeff_189(k-3), coeff_186(k-3),
     &             coeff_184(k-3), coeff_91v(k-3), coeff_91h(k-3)
           ENDIF
        END DO
        GO TO 34
C
   33   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM READING SSMIS IMG COEFFICIENT FILE ',
     &     coeff_file(1:l_cfi),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   34   CONTINUE
        CLOSE(iunit_c)
C
C Read the arrays that were formerly read within ingest-ssmis-tdrr:
C
       !===== Open LUT files for TA matching ====

        coeff_file = 'lut_add_ocean'
        OPEN(iunit_c,FILE=COEFF_FILE, STATUS='OLD', ERR=35)
        GO TO 36
C
   35   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM OPENING SSMIS LUT OCEAN FILE ',
     &     coeff_file(1:13),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   36   CONTINUE
C
        DO i = 1, 161
           READ(iunit_c,FMT='(f6.2,7(f7.2))', ERR=37) ssmis_oce(i),
     &                                              (add_oce(i,j),j=1,7)
        END DO
        GO TO 38
C
   37   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM READING SSMIS LUT OCEAN FILE ',
     &     coeff_file(1:13),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   38   CONTINUE
        CLOSE (iunit_c)

        coeff_file = 'lut_add_land'
        OPEN(iunit_c,FILE=COEFF_FILE, STATUS='OLD',ERR=39)
        GO TO 40
C
   39   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM OPENING SSMIS LUT LAND FILE ',
     &     coeff_file(1:12),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   40   CONTINUE
C
        DO i = 1, 161
           READ(iunit_c,FMT='(f6.2,7(f7.2))', ERR=41) ssmis_lnd(i),
     &                                              (add_lnd(i,j),j=1,7)
        END DO
        GO TO 42
C
   41   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM READING SSMIS LUT LAND FILE ',
     &     coeff_file(1:12),
     &     '; TERMINATING EXECUTION; PROGRAM L3B42MAIN0SS'
        STOP
C
   42   CONTINUE
        CLOSE (iunit_c)
C
C Main processing loop begins here.
C

        DO 80 k = 1, numfiles
C
        linfile = infile(k)
c       read(linfile(80:81), '(i2.2)') iyr
c       read(linfile(65:66), '(i2.2)') iyr
        read(linfile(37:38), '(i2.2)') iyr
        if (iyr.lt.50) then
         iyr = iyr + 2000
        elseif (iyr.gt.50) then
         iyr = iyr + 1900
        endif
        write(6,'(/,''input TDR file year '',i4)') iyr
        write(6,'(/,''input TDR file '',a120)') linfile
c    Create output ascii file using 2nd generation CMORPH standards  

c/cpc/prod_tst/rjoyce/SSMIS_RH6/data/dmsp16/TDRPROC/NPR.TDRN.SA.D14351.S1714.E1858.B5760708.NS
c123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901
c/cpc/prod_tst/rjoyce/SSMIS_RH6/data/dmspID/TDRRAW1/DMSPID__SSMIS_V-GPROF7___YYYYJDY.SHHMM.EHHHH.B###&###.txt

c       output_file = '/cpc/prod_tst/rjoyce/SSMIS_RH6/data/dmspID/TDRRAW
c    &1/DMSPID__SSMIS_V-GPROF7___YYYYJDY.SHHMM.EHHHH.B###&###.txt'
c       write(output_file(41:42),'(i2.2)') id
c       write(output_file(56:57),'(i2.2)') id
c       write (output_file(77:80),'(i4)') iyr
c       output_file(81:83)   = linfile(67:69)
c       output_file(86:89) = linfile(72:75)
c       output_file(92:95) = linfile(78:81)
c       output_file(98:104) = linfile(84:90)

c../data/dmspID/TDRPROC/NPR.TDRN.SA.D14351.S1714.E1858.B5760708.NS
c123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901
c../data/dmspID/TDRRAW1/DMSPID__SSMIS_V-GPROF7___YYYYJDY.SHHMM.EHHHH.B###&###.txt
 
c23456789012345678901234567890123456789012345678901234567890123456789012
       output_file = '../data/dmspID/TDRRAW1/DMSPID__SSMIS_V-GPROF7___YY
     &YYJDY.SHHMM.EHHHH.B###&###.txt'
       write(output_file(13:14),'(i2.2)') id
       write(output_file(28:29),'(i2.2)') id
       write (output_file(49:52),'(i4)') iyr
       output_file(53:55)   = linfile(37:39)
       output_file(58:61) = linfile(42:47)
       output_file(64:67) = linfile(50:53)
       output_file(70:76) = linfile(56:62)


        write(6,'(/,''output decoded file '',a120)') output_file
        open(95,file=output_file,STATUS = 'new')
 
           CALL ST_LSTR (infile(k), l_if, iret)
           IF  (iret .NE. 0) THEN
               IF (do_debug) WRITE (*,*)
     &            'ERROR: UNABLE TO DETERMINE LENGTH OF INPUT ',
     &            'SSMIS FILE NAME ', infile(k), ' ; ADVANCING TO ',
     &            'NEXT FILE; PROGRAM L3B42MAIN0SS'
               GO TO 80
           END IF
C
 
	   CALL RETRIEVAL(cjobid, infile(k), l_if, iunit_t,
     &                    outpath_g, coeff_19v, coeff_19h, coeff_22v,
     &                    coeff_37v, coeff_37h, coeff_91v, coeff_91h,
     &                    add_oce, add_lnd, nsec0, nsec1, sat_name,
     &                    do_debug)
 
C
C Loop back up to get the next input file from the list file.
C
        close(95)
   80   END DO
c       close(95)
        CLOSE(iunit_t)
        STOP
        END
	SUBROUTINE ST_CLST  ( string, sep, cdef, nexp, carr, num, iret )
C************************************************************************
C* ST_CLST								*
C*									*
C* This subroutine breaks a string containing a list of strings into	*
C* an array of strings.  The separator for the strings is input as SEP.	*
C* If the separator is a blank, multiple blanks will be changed to	*
C* single blanks before the string is processed.  If null strings	*
C* are encountered or fewer than NEXP strings are found in the		*
C* string, the appropriate CARR locations are set to CDEF.		*
C*									*
C* ST_CLST  ( STRING, SEP, CDEF, NEXP, CARR, NUM, IRET )		*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String 				*
C*	SEP		CHAR*1		Separator			*
C*	CDEF		CHAR*		Default string 			*
C*	NEXP		INTEGER 	Number of expected values 	*
C*									*
C* Output parameters:							*
C*	CARR  (NUM)	CHAR*		Array of strings 		*
C*	NUM		INTEGER 	Number of strings returned	*
C*	IRET		INTEGER 	Return code			*
C*				   	  1 = more than NEXP values	*
C*				   	  0 = normal return		*
C**									*
C* Log:									*
C* M. Goodman/RDS	10/84	Original source for STLIST		*
C* M. desJardins/GSFC	11/84						*
C* M. desJardins/GSFC	 2/85	Modified for ST_CLST			*
C* M. desJardins/GSFC	 6/88	Documentation				*
C************************************************************************
 	CHARACTER*(*) 	string, sep, carr (*), cdef
C*
	CHARACTER	strbuf*132, cchar*1
C------------------------------------------------------------------------
	iret = 0
	num  = 0

C
C*	Remove blanks from the input string if the separator is not
C*	a blank.
C

	IF  ( sep .ne. ' ' )  THEN
	    CALL ST_RMBL  ( string, strbuf, isize, iret )
	  ELSE
	    CALL ST_RXBL  ( string, strbuf, isize, iret )
	END IF

C*	Initialize output array.
C
	DO  i = 1, nexp
	    carr (i) = cdef
	END DO

C
C*	Check for zero length input string.
C
	IF  ( isize .eq. 0 )  THEN
	    num = 0
C
C*	    Check for separator and find list elements.
C
	  ELSE
	    cchar  = sep
	    iend   = 0
	    ibegin = 1
	    DO WHILE  ( ibegin .le. isize )
	        loc = INDEX  ( strbuf ( ibegin: ), cchar )
	        IF  ( loc .eq. 0 )  THEN
		    iend = isize + 1
	          ELSE
		    iend = ibegin + loc - 1
	        END IF
C
C*	        Add into output list.  Check that num <= nexp.
C
	        IF  ( num .ge. nexp )  THEN
		    iret = 1
		  ELSE
		    num = num + 1
	    	    IF  ( ibegin .ne. iend )  THEN
			carr ( num ) = strbuf ( ibegin : iend-1 )
	    	    END IF
		END IF
	    ibegin = iend + 1
	    END DO
	END IF

C*
	RETURN
	END
	SUBROUTINE ST_LDSP  ( string, outstr, ncout, iret )
C************************************************************************
C* ST_LDSP								*
C*									*
C* This subroutine deletes the leading spaces and tabs in a string.	*
C* The input and output strings may be the same variable.		*
C*									*
C* ST_LDSP  ( STRING, OUTSTR, NCOUT, IRET )				*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String				*
C*									*
C* Output parameters:							*
C*	OUTSTR		CHAR*		Output string			*
C*	NCOUT		INTEGER		Number of characters output	*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C**									*
C* Log:									*
C* I. Graffman/RDS	 2/84	Use new GEMPAK routines			*
C* M. desJardins/GSFC	11/84	Fixed					*
C* M. desJardins/GSFC	 6/88	Documentation				*
C************************************************************************
	INCLUDE		'GEMINC:GEMPRM.PRM'
C*
	CHARACTER*(*)	string, outstr
C*
	CHARACTER	stbuf*132, c*1
C*-------------------------------------------------------------------------
	stbuf = string
	iret  = 0
C
C*	Get length of string.
C
	CALL ST_LSTR  ( stbuf, lens, iret )
C
C*	If length is non-zero, find first non space.
C
	IF  ( lens .eq. 0 )  THEN
	    ncout  = 0
	    outstr = ' '
	  ELSE
	    jp = 1
	    c  = stbuf ( jp:jp )
C
 	    DO WHILE  ( ( ( c .eq. CHSPAC ) .or. ( c .eq. CHTAB ) .or.
     +			  ( c .eq. CHNULL ) ) .and. ( jp .le. lens ) )
 		jp = jp + 1
 		IF  ( jp .le. lens )  c = stbuf ( jp:jp )
 	    ENDDO
C
C*	    Compute length and fill output string.
C
	    ncout = lens - jp + 1
	    IF  ( ncout .gt. 0 )  THEN
		outstr = stbuf ( jp : lens )
	      ELSE
		outstr = ' '
	    END IF
	ENDIF
C*  
	RETURN
	END
	SUBROUTINE ST_LSTR  ( string, lens, iret )
C************************************************************************
C* ST_LSTR								*
C*									*
C* This subroutine returns the number of characters in a string 	*
C* disregarding trailing null characters, tabs and spaces.		*
C*									*
C* ST_LSTR  ( STRING, LENS, IRET )						*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String 				*
C*									*
C* Output parameters:							*
C*	LENS		INTEGER 	Length of string		*
C*	IRET		INTEGER		Return code			*
C*				 	 0 = normal return 		*
C**									*
C* Log:									*
C* J. Woytek/GSFC	 6/82 	STR_LNSTR				*
C* I. Graffman/RDS	 2/84 	Fix zero length string handling		*
C* M. desJardins/GSFC	 6/88	Rewritten				*
C************************************************************************
	INCLUDE		'GEMINC:GEMPRM.PRM'
C*
	CHARACTER*(*)	string
C*
	CHARACTER*1	c
C*------------------------------------------------------------------------
	lens = 0
	iret = 0
C
C*	Get the actual length of the string.
C
	lens = LEN  ( string )
	IF  ( lens .eq. 0 )  RETURN
C
C*	Start at last character and loop backwards.
C
	ip = lens
	DO WHILE  ( ip .gt. 0 )
C
C*	    Get current value of string and check for space, null, tab.
c
	    c = string ( ip : ip )
 	    IF  ( ( c .eq. CHSPAC ) .or. ( c .eq. CHNULL ) .or.
     +		  ( c .eq. CHTAB  ) )  THEN
 		lens = lens - 1
 		ip   = ip - 1
 	      ELSE
 		ip   = 0
 	    END IF
	END DO
C*
	RETURN
	END
	SUBROUTINE ST_NUMB  ( string, ival, iret )
C************************************************************************
C* ST_NUMB                                       			*
C*									*
C* This subroutine converts a string into an integer.			* 
C*									*
C* ST_NUMB  ( STRING, IVAL, IRET )					*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String				*
C*									*
C* Output parameters:							*
C*	IVAL		INTEGER		Integer value      		*
C*	IRET		INTEGER		Return code			*
C*				 	  0 = normal return		*
C*					 -2 = conversion error		*
C** 									*
C* Log:									*
C* M. desJardins/NMC	 3/92	Rewritten to avoid special cases	*
C************************************************************************
	INCLUDE		'GEMINC:GEMPRM.PRM'
C*
	CHARACTER*(*) 	string
C*
	CHARACTER	sss*12, c*1
	LOGICAL		good, plus
C------------------------------------------------------------------------
	iret = -2
	ival = IMISSD
C
C*	Remove blanks from string.
C
	CALL ST_RMBL  ( string, sss, lens, ier )
C
C*	Check for empty string.
C
	IF  ( lens .eq. 0 )  RETURN
C
C*	If last character is period, remove it.
C
	IF  ( sss (lens:lens) .eq. '.' )  THEN
	    sss (lens:lens) = ' '
	    lens = lens - 1
	END IF
	IF  ( lens .eq. 0 )  RETURN
C
C*	Check for + or - in first character.
C
	IF  ( sss (1:1) .eq. '+' )  THEN
	    ibeg = 2
	    plus = .true.
	  ELSE IF  ( sss (1:1) .eq. '-' )  THEN
	    ibeg = 2
	    plus = .false.
	  ELSE
	    ibeg = 1
	    plus = .true.
	END IF
	IF  ( ibeg .gt. lens )  RETURN
C
C*	Now loop through all characters and turn into integer.
C
	ival0 = ICHAR ( '0' )
	ival  = 0
	good  = .true.
	i     = ibeg
	DO  WHILE  ( good .and. ( i .le. lens ) )
	    c     = sss (i:i)
	    ivalc = ICHAR ( c ) - ival0
	    IF  ( ( ivalc .ge. 0 ) .and. ( ivalc .le. 9 ) )  THEN
		ival  = ival * 10 + ivalc
		i     = i + 1
	      ELSE
		good  = .false.
	    END IF
	END DO
C
C*	Check for good value and add sign.
C
	IF  ( .not. good )  THEN
	    ival = IMISSD
	  ELSE IF  ( plus )  THEN
	    iret = 0
	  ELSE
	    iret = 0
	    ival = -ival
	END IF
C*
	RETURN
	END
	SUBROUTINE ST_RMBL  ( string, outstr, length, iret )
C************************************************************************
C* ST_RMBL								*
C*									*
C* This subroutine removes spaces and tabs from a string.  The input	*
C* and output strings may be the same variable.				*
C*									*
C* ST_RMBL  ( STRING, OUTSTR, LENGTH, IRET )				*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String				*
C*									*
C* Output parameters:							*
C*	OUTSTR		CHAR*		String without blanks		*
C*	LENGTH		INTEGER		Length of output string		*
C*	IRET		INTEGER		Return code			*
C*				   	 0 = normal return 		*
C**									*
C* Log:									*
C* M. desJardins/GSFC	 8/84						*
C* M. desJardins/GSFC	 6/88	Documentation				*
C* M. desJardins/NMC	 3/92	Add temporary variable ttt		*
C************************************************************************
	INCLUDE		'GEMINC:GEMPRM.PRM'
C*
	CHARACTER*(*)	string, outstr
C*
	CHARACTER	c*1, sss*132, ttt*132
C-----------------------------------------------------------------------
	iret   = 0
	length = 0
	sss    = string
	ttt    = ' '
C
C*	Get length of input string.
C
	CALL ST_LSTR  ( sss, lens, iret )
C
C*	Check each character to see if it is a blank.
C
	DO  i = 1, lens
	    c = sss (i:i)
 	    IF  ( ( c .ne. CHSPAC ) .and. ( c .ne. CHTAB ) )  THEN
 		length = length + 1
 		ttt ( length : length ) = c
 	    END IF
	END DO
C*
	outstr = ttt
C*
	RETURN
	END
	SUBROUTINE ST_RXBL  ( string, outstr, length, iret )
C************************************************************************
C* ST_RXBL								*
C*									*
C* This subroutine removes extra spaces and tabs from a string.  Only	*
C* single blanks will separate substrings.  The input and output 	*
C* strings may be the same variable.					*
C*									*
C* ST_RXBL  ( STRING, OUTSTR, LENGTH, IRET )				*
C*									*
C* Input parameters:							*
C*	STRING		CHAR*		String				*
C*									*
C* Output parameters:							*
C*	OUTSTR		CHAR*		String without blanks		*
C*	LENGTH		INTEGER		Length of output string		*
C*	IRET		INTEGER		Return code			*
C*				   	 0 = normal return 		*
C**									*
C* Log:									*
C* M. desJardins/GSFC	 9/88						*
C************************************************************************
	INCLUDE		'GEMINC:GEMPRM.PRM'
C*
	CHARACTER*(*)	string, outstr
C-----------------------------------------------------------------------
	length = 0
	iret   = 0
C
C*	Remove leading spaces and tabs.
C
	CALL ST_LDSP  ( string, outstr, isiz, iret )
	IF  ( isiz .le. 0 )  RETURN
C
C*	Remove extra spaces.
C
	ispac  = 0
	length = 0
	DO  j = 1, isiz
 	    IF  ( ( outstr (j:j) .ne. CHSPAC )  .and. 
     +		  ( outstr (j:j) .ne. CHTAB  ) )  THEN
 	        length = length + 1
 	        outstr (length:length) = outstr (j:j)
 	        ispac = 0
 	      ELSE
 	        IF  ( ispac .eq. 0 )  THEN
 	            length = length + 1
 	            outstr (length:length) = ' '
 	            ispac = 1
 	        END IF
 	    END IF
	END DO
C
C*	Make sure the end of the string is blank.
C
	lens = LEN ( outstr )
	IF  ( lens .gt. length )  outstr ( length+1 : ) = ' '
C*
	RETURN
	END
