      subroutine convtime_wentz(nwork, wentz_time, stdtime)
      implicit none
      include 'parameters.inc'

      integer     nwork
      integer     wentz_time(MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)

      integer     lin, i
      integer*2   time(6)                 ! YYYY MM DD HH MM SS

      do lin = 1, nwork
            call wentz_to_std(wentz_time(lin), time)

            do i = 1, 6
                  stdtime(lin, i) = time(i)
            end do
      end do

      return
      end

C ----- Time Conversion Subroutines and Functions -----


      subroutine wentz_to_std(sec70, time)
      implicit none
      integer*4   sec70
      integer*2   time(6)

      integer*4   sec_per_day, sec_per_hour, sec_per_min
      parameter   (sec_per_day = 86400, sec_per_hour = 3600,
     &             sec_per_min = 60)

      integer*4   sec
      integer*2   year, month

      integer*4   sec_in_year, sec_in_month
      external    sec_in_year, sec_in_month

      sec = sec70

      year = 1970
      do while ((sec - sec_in_year(year)) .GE. 0)
            sec = sec - sec_in_year(year)
            year = year + 1
      end do
      time(1) = year

      month = 1
      do while ((sec - sec_in_month(month,year)) .GE. 0)
            sec = sec - sec_in_month(month,year)
            month = month + 1
      end do
      time(2) = month

      ! days
      time(3) = sec / sec_per_day + 1
      sec = mod(sec, sec_per_day)

      ! hours
      time(4) = sec / sec_per_hour
      sec = mod(sec, sec_per_hour)

      ! minutes
      time(5) = sec / sec_per_min
      sec = mod(sec, sec_per_min)

      ! seconds
      time(6) = sec

      return
      end



      integer*4 function sec_in_year(year)
      implicit none
      integer*2   year

      logical           isleapyr
      external    isleapyr

      if (isleapyr(year)) then
            sec_in_year = 31622400
      else
            sec_in_year = 31536000
      end if

      return
      end


      integer*4 function sec_in_month(month, year)
      implicit none
      integer*2   month, year

      integer*4   sec_per_day
      parameter   (sec_per_day = 86400)

      integer           days(12)

      logical           isleapyr
      external    isleapyr

      data days   / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

      if (month .NE. 2) then
            ! All months except February
            sec_in_month = days(month) * sec_per_day
      else
            ! February
            if (isleapyr(year)) then
                  sec_in_month = 29 * sec_per_day
            else
                  sec_in_month = 28 * sec_per_day
            end if
      end if

      return
      end


      logical function isleapyr(year)
      implicit none
      integer*2   year

      if ( ((mod(year, 4) .EQ. 0) .AND. (mod(year, 100) .NE. 0)) .OR.
     &      (mod(year, 400) .EQ. 0) ) then
            isleapyr = .TRUE.
      else
            isleapyr = .FALSE.
      end if

      return
      end
