	SUBROUTINE CONVTIME (juldate, secsoday, isecs)
C
C***********************************************************************
C* CONVTIME  	Converts time in julian date and seconds of day	to     *
C*		seconds since 00:00:00 UTC 1/1/1987		       *
C*								       *
C* Called By:	INGEST_SDR					       *
C*								       *
C* Calls:	<none>						       *
C*  								       *
C* CONVTIME converts time from julian date and seconds of day to       *
C* seconds since 00:00:00 UTC 01/01/1987.			       *
C*  								       *
C* Input Parameters:						       *
C*								       *
C*  juldate	INT	packed julian date (YYYYDDD)		       *
C*  secsoday	REAL	time in seconds of day beginning at the start  *
C*			of the current day			       *
C* 								       *
C* Output Parameters:						       *
C*								       *
C*  isecs	INT	seconds since 00:00:00 UTC on 01/01/1987       *
C* 								       *
C**								       *
C* Log:								       *
C* D.Bolvin/SSAI	 9/97	First Version			       *
C* D.Bolvin/SSAI	10/97	Return negative seconds of day         *
C*				(missing) with seconds since 00:00:00  *
C*				UTC on 01/01/1987		       *
C***********************************************************************
C
	IMPLICIT	NONE
C
C***********************************************************************
C	Declaration of subroutine input/output arguments.
C
	INTEGER*4	juldate, isecs
C
	REAL*4		secsoday
C
C***********************************************************************
C	Declaration of internal subroutine variables.
C
	INTEGER*4	iyear, jday, iydiff, ipysecs, icysecs, 
     +			IDINY/365/, ISIND/86400/
C
C***********************************************************************
C	Extract the year and julian day from juldate.
C
	iyear = juldate / 1000
	jday = juldate - (iyear * 1000) 
C
C***********************************************************************
C	Compute the number of seconds since 00:00:00 UTC 01/01/1987 up
C	to the beginning of the current year.  Include leap days.
C
	iydiff = (iyear - 1987)
C
	ipysecs = ((iydiff * IDINY) + ((iydiff + 2) / 4)) * ISIND
C
C***********************************************************************
C	Compute the number of seconds since the beginning of the current
C	year up to the current time.
C
	icysecs = ((jday - 1) * ISIND) + IABS (NINT (secsoday))
C
C***********************************************************************
C	Compute the number of seconds since 00:00:00 UTC 01/01/1987 up
C	to the current time.
C
	isecs = ipysecs + icysecs
C
	RETURN
	END
