
      ! Ye Hong  05/21/97   a. Fix a bug occurred when lat. & lon. are missing
      !                     b. Rearrange lines to avoid "goto" statement.
      ! Ye Hong  09/08/97   a. integer*2   sfctype ==> integer*1   sfctype
      !                     b. stdtime(..., 7) ==> stdtime(..., 6)
      !                     c. Added checking indices of row and col 
      !                     d. nint(Tc(pix,lin,2) ==> nint(Tc(pix,lin,4) 
      !                        nint(Tc(pix,lin,3) ==> nint(Tc(pix,lin,5) 
      ! Ye Hong 09/26/97    0. and 131. ==> 0 and 131 in if statement
      ! Ye Hong 10/30/97    a. TTW freezing level table for TMI has been updated.  FL
      !                        will be zero if routine fit.f cannot be converged.  So
      !                        that one more if statement is added.
      !                     b. Added 500m to climatological FL (according CK on 10/29)
      ! Ye Hong 6/10/99     a. Added output sst
      ! Ye Hong 7/30/99     a. Removed output sst
      ! W. Olson 7/4/02	    a. Minimum variance composite of spatial-mean
      ! 		       Wilheit estimates and climatology
      ! H. Masunaga 12/9/02 a. Added freezing height interpolation subroutine
      ! J. Thomas-Stahle 1/03 a. Modified the retrieved freezing height
      !                        below 4500 m.     
      
      ! Code first assigns Wilheit's freezing level to ocean
      ! pixels. The freezing levels are derived from brightness  
      ! temperatures of 19V and 22V channels (Whilheit et al, 1991).
      ! A fixed freezing level is used otherwise.
      
      subroutine frzhgt_ttw(nwork, Tc, flttw_tab, sstdata, 
     &           pixel_status, lat, lon, stdtime, sfctype, 
     &           frzheight)
      
      implicit none
      include 'parameters.inc'

      integer     nwork
      integer*2   sstdata(0:143,0:90,12)
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*1   sfctype(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)  ! YYYY MM DD HH MM SS
      real        flttw_tab(131, 131)
      real        Tc(NPIX, MAX_SCANS, NCHAN) ! K
      real        lat(NPIX, MAX_SCANS)   ! [-90, +90]
      real        lon(NPIX, MAX_SCANS)   ! [-180, +180]
      real        frzheight(NPIX, MAX_SCANS)   ! km

      integer     lin, pix, bt19v, bt22v
      integer     lintemp
      integer     month, row, col
      real        sst(NPIX, MAX_SCANS)

    ! New variables for minimum variance estimate
      integer     nrun
      integer	  n_frz_w_mean(MAX_SCANS)
      integer     n_frz_c_mean(MAX_SCANS)
      integer     ll
      integer     num_frz_w_mean
      integer     num_frz_c_mean

      real        frz_w(NPIX, MAX_SCANS)
      real        frz_c(NPIX, MAX_SCANS)
      real        frz_w_mean(MAX_SCANS)
      real        frz_c_mean(MAX_SCANS)
      real        sum_frz_w_mean
      real        sum_frz_c_mean
      real        sig_w
      real        sig_c
      real        wgt_w
      real        wgt_c
      real        scan_wgt
      real        sum_scan_wgt_w
      real        sum_scan_wgt_w2
      real        sum_scan_wgt_c

      data        nrun/27/
      data        sig_w/1000./
      data        sig_c/1000./

      ! ADDED BY HIRO
      real        frzheight_int(NPIX, MAX_SCANS)   ! km
      real        frz_nomean(NPIX,MAX_SCANS)

cejn debug
c     do lin = 1, 4000
c        write(512,*) lin, stdtime(lin,2), lat(1,lin), lon(1,lin)
c     end do
      do lin = 1, nwork
         do pix = 1, NPIX

    ! Account for lack of timing/lat/lon information
            lintemp=lin
            if(lat(pix,lin) .lt. -9990.) then
              if(lin .lt. 1) lintemp=1
              if(lin .gt. nwork) lintemp=nwork
            end if

            month = stdtime(lintemp,2)
            row = int((lon(pix,lintemp) + 180.0)/2.5 )
            col = int((lat(pix,lintemp) + 90.0)/2.0)
            if(row .gt. 143)then
               row = 143
            else if(row .lt. 0)then
               row = 0
            endif
            if(col .gt. 90)then
               col = 90
            else if(col .lt. 0)then
               col = 0
            endif
cejn debug
c           if (pix.eq.1) write(*,*) 'lin/lintemp/month/row/col = ',
c    &         lin, lintemp, month, row, col
            sst(pix,lin) = 0.1*sstdata(row, col, month)

            frz_w(pix,lin)=-999.
            frz_c(pix,lin)=-999.

          ! Climatology
            frz_c(pix,lin) = 500. +
     &       (sst(pix,lin) - 273.15)/0.0065

            ! Skip if pixel is not O.K.           
            if ( pixel_status(pix,lin) .ge. 0)then

               bt19v = nint(Tc(pix,lin,2) - 175.)
               bt22v = nint(Tc(pix,lin,3) - 175.)
            
             ! Wilheit
               if((sfctype(pix,lin) .eq. 0) .and. (bt19v .gt. 0) .and.
     $            (bt22v .gt. 0) .and. (bt19v .le. 131) .and.
     $            (bt22v .le. 131)) then
                  frz_w(pix,lin) = flttw_tab(bt22v,bt19v)*1000.
                 if(frz_w(pix,lin) .lt. 4500)
     $             frz_w(pix,lin) = (frz_w(pix,lin)*1.333) - 1500.
    !             answer of 0 indicates no estimate
                  if(frz_w(pix,lin) .lt. 1.e-5) frz_w(pix,lin)=-999.
               endif

            endif  ! end of if( pixel_status(pix,lin) .ge. 0)

            frz_nomean(pix,lin) = frz_c(pix,lin)
            if (frz_w(pix,lin).gt.0.)
     &      frz_nomean(pix,lin) = frz_w(pix,lin)

         end do

    ! Average estimates in each scan line
         n_frz_w_mean(lin)=0
         frz_w_mean(lin)=0.
         n_frz_c_mean(lin)=0
         frz_c_mean(lin)=0.

         do pix = 1, NPIX
           if(frz_w(pix,lin) .gt. -990.) then
             n_frz_w_mean(lin)=n_frz_w_mean(lin)+1
             frz_w_mean(lin)=frz_w_mean(lin)+frz_w(pix,lin)
           end if 
           if(frz_c(pix,lin) .gt. -990.) then
             n_frz_c_mean(lin)=n_frz_c_mean(lin)+1
             frz_c_mean(lin)=frz_c_mean(lin)+frz_c(pix,lin)
           end if 
         end do

         if(n_frz_w_mean(lin) .gt. 0) then
           frz_w_mean(lin)=frz_w_mean(lin)/float(n_frz_w_mean(lin))
         else
           frz_w_mean(lin)=-999.
         end if
         if(n_frz_c_mean(lin) .gt. 0) then
           frz_c_mean(lin)=frz_c_mean(lin)/float(n_frz_c_mean(lin))
         else
           frz_c_mean(lin)=-999.
         end if

      end do 

    ! Take running means and compose minimum variance estimate
    ! of freezing level
      do lin = 1, nwork

        num_frz_w_mean=0
        sum_scan_wgt_w=0.
        sum_scan_wgt_w2=0.
        sum_frz_w_mean=0.
        num_frz_c_mean=0
        sum_scan_wgt_c=0.
        sum_frz_c_mean=0.

        do ll = lin-nrun, lin+nrun

          scan_wgt=1.-abs(float(ll-lin))/float(nrun)

          if(ll .ge. 1 .and. ll .le. nwork) then

            if(frz_w_mean(ll) .gt. -990.) then
              num_frz_w_mean=num_frz_w_mean+n_frz_w_mean(ll)
              sum_scan_wgt_w=sum_scan_wgt_w+scan_wgt*
     &         float(n_frz_w_mean(ll))
              sum_scan_wgt_w2=sum_scan_wgt_w2+(scan_wgt*
     &         float(n_frz_w_mean(ll)))**2.
              sum_frz_w_mean=sum_frz_w_mean+scan_wgt*
     &         float(n_frz_w_mean(ll))*frz_w_mean(ll)
            end if
            if(frz_c_mean(ll) .gt. -990.) then
              num_frz_c_mean=num_frz_c_mean+n_frz_c_mean(ll)
              sum_scan_wgt_c=sum_scan_wgt_c+scan_wgt*
     &         float(n_frz_c_mean(ll))
              sum_frz_c_mean=sum_frz_c_mean+scan_wgt*
     &         float(n_frz_c_mean(ll))*frz_c_mean(ll)
            end if

          end if

        end do

        if(sum_scan_wgt_w .gt. 0) then
          sum_frz_w_mean=sum_frz_w_mean/(sum_scan_wgt_w)
          wgt_w=(sum_scan_wgt_w*sum_scan_wgt_w)/
     &     (sum_scan_wgt_w2*sig_w*sig_w)
        else
          sum_frz_w_mean=-999.
          wgt_w=0.
        end if
        if(num_frz_c_mean .gt. 0) then
          sum_frz_c_mean=sum_frz_c_mean/(sum_scan_wgt_c)
          wgt_c=1./(sig_c*sig_c)
        else
          sum_frz_c_mean=-999.
          wgt_c=0.
        end if

        if(sum_frz_w_mean .gt. -990. .or.
     &     sum_frz_c_mean .gt. -990.) then

    ! Valid estimates found
          do pix=1,NPIX
            if(pixel_status(pix,lin) .ge. 0)

     &       frzheight(pix,lin)=(wgt_w*sum_frz_w_mean+
     &                           wgt_c*sum_frz_c_mean)/
     &                          (wgt_w+wgt_c)


c    ! Constrain the freezing level to a sst of 302 K
c             if ( frzheight(pix,lin) .gt. 4938. )
c     $            frzheight(pix,lin) = 4938.

          end do

        end if


      end do
           
      call int_frzhgt(nwork, frz_c, frz_w, frzheight)

      return
      end
