	SUBROUTINE MK_ONAME  ( swathfile, sat_name, istime, ietime,
     &                         oname, iret )
C************************************************************************
C* MK_ONAME  Create a local output filename for an SSMIS swath		*
C* Input "swathfile" is a CLASS TDR (Ta) file.                          *
C*									*
C* This subroutine uses the SSMIS level 2 swathfile name, and start and *
C* stop times to build a level 3 file name of the form			*
C*   GPROF2004V.<satellite_id>.<start_date>_<start_time>_<end_date>_    *
C*                                                           <end_time> *
C* where								*
C*   <satellite_id> = 2-character satellite id (e.g., "F17")        	*
C*   <start_date>   = YYYYMMDD                                          *
C*   <start_time>   = HHMMSS                                            *
C*   <end_date>     = YYYYMMDD						*
C*   <end_time>     = HHMMSS						*
C*									*
C* MK_ONAME  ( swathfile, istime, ietime, oname, iret )			*
C*									*
C* Input parameters:							*
C*   swathfile	CHAR*	L.2 SSMIS orbit file name			*
C*   sat_name   CHAR*3  satellite ID (e.g., "F17")                      *
C*   istime	INT4*6	Date/time of orbit start			*
C*   ietime	INT4*6	Date/time of orbit stop				*
C*									*
C* Output parameters:							*
C*   oname	CHAR*	L.3 SSMIS grid file name			*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* E.Nelkin/SSAI         5/11  Based on AMSU-B routine used in TRMM V6  *
C* E.Nelkin/SSAI        10/11  Use capital "F" in satellite ID          *
C* E.Nelkin/SSAI        11/11  Pass in sat_name rather than building it *
C*                             from swathfile name                      *
C************************************************************************
	IMPLICIT	NONE
        INCLUDE         'parameters.inc'
C
	INTEGER*4	isat, ival, j
C       INTEGER*4       iorbit
        CHARACTER*4     cyyyy
	CHARACTER*2	cmm, cdd, chh, css
        CHARACTER*1     aloop
C       CHARACTER*5     aorbit
C
	CHARACTER*256	swathfile, oname
        CHARACTER*3     sat_name
	INTEGER*4	istime (6), ietime (6), iret
C       SAVE            aorbit
C
	iret = 0
C
C       Iniitalize character strings to blanks.
C
        do j = 1, 4
           cyyyy(j:j) = ' '
        end do
        do j = 1, 2
           cmm(j:j) = ' '
        end do
        cdd = cmm
        chh = cmm
        css = cmm
C
C	Set 'GPROF2004V.' at start of name (note: a local file), then
C	index into swathname to get satellite name.  The swathname
C	might have a leading directory structure, but the actual file 
C	name is a fixed format
C	  NPR.TDRN.<ss>.D<yy><ddd>.S<hh><mm>.E<hh><mm>.B<o1><o2>.NS
C	in which ss is the 2-character satellite ID and B<o1><o2> is the
C	7-character NESDIS-format orbit ID (see header, above).
CC
CC      11/2011 - Use the sat_name as passed in, rather than obtaining
CC      the satellite ID from the swathfile name.  This will allow for
CC      flexibility for future SSMIS instruments, since the ID will
CC      come directly from the run script, obviating the need to modify
CC      the (now "CC"-commented) IF test below.
C
C	Append the satellite ID, then convert start and end date and
C	time to characters, and append.
C
	oname (1:11)   = 'GPROF2004V.'
        oname(12:15)   = sat_name(1:3) // '.'
CC	ival           = INDEX ( swathfile, 'TDRN' )
CC      isat = ival + 6
C
CC      IF ( swathfile(isat:isat) .eq. 'A' ) THEN
CC         oname(12:15) = 'F16.'
CC      ELSE IF ( swathfile(isat:isat) .eq. 'B' ) THEN
CC         oname(12:15) = 'F17.'
CC      ELSE IF ( swathfile(isat:isat) .eq. 'C' ) THEN
CC         oname(12:15) = 'F18.'
CC      ENDIF 
C
	WRITE (cyyyy, 10) istime (1)
   10   FORMAT ( I4.4 )
	WRITE (cmm,  11) istime (2)
   11   FORMAT ( I2.2 )
	WRITE (cdd,  11) istime (3)
	oname (16:24) = cyyyy // cmm // cdd // '_'
	WRITE (chh,  11) istime (4)
	WRITE (cmm,  11) istime (5)
	WRITE (css,  11) istime (6)
        oname (25:31) = chh // cmm // css // '_'
        WRITE (cyyyy, 10) ietime (1)
        WRITE (cmm, 11) ietime (2)
        WRITE (cdd, 11) ietime (3)
        oname (32:40) = cyyyy // cmm // cdd // '_'
        WRITE (chh, 11) ietime (4)
        WRITE (cmm, 11) ietime (5)
        WRITE (css, 11) ietime (6)
        oname (41:46) = chh // cmm // css
C
	RETURN
	END
