      ! Write retrieved quantities to output file for analysis.

      subroutine output_pathfinder(nwork, lat, lon, stdtime,
     $               pixel_status, rain_status, sfcrain, convect,
     $               quality, lyrhgt, clw, rain, ciw, ice, 
     $               start_time, sum_sfcrain, sum_ncount, 
     $               sum_nambig, sum_intliq, sum_intice, 
     $               sum_confdce, sum_convect, sum_nrpix, grid_time)

      implicit none

      include 'parameters.inc'


      integer     nwork
      real        lat(NPIX, MAX_SCANS)
      real        lon(NPIX, MAX_SCANS)
      real        Tb (NPIX, MAX_SCANS, NCHAN)
      real        Tc (NPIX, MAX_SCANS, NCHAN)
      integer*2   stdtime(MAX_SCANS, 6)
      real        frzheight(NPIX, MAX_SCANS) 
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)
      real        sfcwind(NPIX, MAX_SCANS)      ! m/s
      real        varblty(NPIX, MAX_SCANS)
      real        fov_cover(NPIX, MAX_SCANS)
      real        sfcrain(NPIX, MAX_SCANS)
      real        convect(NPIX, MAX_SCANS)
      real        quality(NPIX, MAX_SCANS)
      real        lyrhgt(NPIX, MAX_SCANS, NLYR)
      real        clw (NPIX, MAX_SCANS, NLYR)
      real        rain(NPIX, MAX_SCANS, NLYR)
      real        ciw (NPIX, MAX_SCANS, NLYR)
      real        ice (NPIX, MAX_SCANS, NLYR)

      ! Area averaged output variables
      integer     start_time(6)
      real        sum_sfcrain(360,720)
      integer     sum_ncount(360,720)
      integer     sum_nambig(360,720)
      real        sum_intliq(360,720) 
      real        sum_intice(360,720)
      real        sum_confdce(360,720)
      real        sum_convect(360,720)
      integer     sum_nrpix(360,720)
      integer*1   grid_time(360,720)

      integer     lin, pix, lyr, chan, i, ilat, ilon
      integer     index_start, index_line
      integer     scan_minutes, start_minutes
      real        dz, longitude
      real        tot_liq(NPIX, MAX_SCANS)
      real        tot_ice(NPIX, MAX_SCANS)

c
c     Start time is the first time in the orbit.
      do i = 1, 6
        start_time(i) = stdtime(1,i)
      end do
c
c     Start minutes is the time in minutes of the first scan
c     
      start_minutes = (start_time(4) * 60) + start_time(5)
c
c
c     Compute the integrated liq and ice for each pixel
c
CSPD      do lin = 1, nwork
CSPD         do pix = 1, NPIX
CSPD            do lyr = 1, NLYR
CSPD               if ( lyr .eq. 1 ) then
CSPD                 dz = lyrhgt(pix,lin,lyr)
CSPD               else
CSPD                 dz = lyrhgt(pix,lin,lyr) - lyrhgt(pix,lin,lyr-1)
CSPD               endif
CSPD               tot_liq(pix,lin)  = tot_liq(pix,lin)  + dz*
CSPD&                   ( clw(pix,lin,lyr) + rain(pix,lin,lyr))
CSPD               tot_ice(pix,lin)  = tot_ice(pix,lin)  + dz*
CSPD&                   ( ciw(pix,lin,lyr) + ice(pix,lin,lyr))
CSPD            end do
CSPD         end do
CSPD      end do 

c     Accumulate parameters into 0.5 deg. grid elements

      do lin = 1, nwork
         ! scan minutes is the time in minutes of the current scan
         scan_minutes = stdtime(lin,4)*60 + stdtime(lin,5)
         do pix = 1, NPIX

            ! skip to end if pixel_status < 0
            if ( pixel_status(pix,lin) .lt. 0) goto 10

            ! if sfcrain < 0., then also skip to end
            if ( sfcrain(pix,lin) .lt. 0) goto 10


            ilat = int(180. - 2.*lat(pix,lin)) + 1
            if ( ilat .gt. 360 ) ilat = 360
            if ( lon(pix,lin) .lt. 0.) then
              longitude = 360 + lon(pix,lin)
            else
              longitude = lon(pix,lin)
            endif
            ilon = int(2.*longitude) + 1
            if ( ilon .gt. 720) ilon = 720


    ! Trap for eventual assignment to byte variable
    ! Do not accumulate more than 255 estimates
            if(sum_ncount(ilat,ilon) .eq. 255) go to 10


            sum_ncount(ilat,ilon) = sum_ncount(ilat,ilon) + 1
            if ( sfcrain(pix,lin).gt.0 )
     $         sum_nrpix(ilat,ilon) = sum_nrpix(ilat,ilon) + 1
c
c           Keep track of the time offset in minutes from the starting
c           scan time for each grid box.  This grid will always contain the
c           last "time" that a data point was accumulated.
c            
	    grid_time(ilat,ilon) = scan_minutes - start_minutes
 
c           If day was about to change, grid time can be negative.

	    if ( scan_minutes .lt. start_minutes ) 
     $         grid_time(ilat,ilon) = 24 * 60 + scan_minutes - 
     $                                start_minutes

            if ((rain_status(pix,lin) .gt.   0) .or.
     $          (rain_status(pix,lin) .eq. -31))  
     $         sum_nambig(ilat,ilon) = sum_nambig(ilat,ilon) + 1

            if ( rain_status(pix,lin) .ge. 0 ) then
              sum_sfcrain(ilat,ilon) = sum_sfcrain(ilat,ilon) +
     $             sfcrain(pix,lin)
CSPD              sum_intliq(ilat,ilon) = sum_intliq(ilat,ilon) +
CSPD     $             tot_liq(pix,lin)
CSPD              sum_intice(ilat,ilon) = sum_intice(ilat,ilon) +
CSPD     $             tot_ice(pix,lin)
CSPD              sum_convect(ilat,ilon) = sum_convect(ilat,ilon) +
CSPD     $             convect(pix,lin)
              sum_confdce(ilat,ilon) = sum_confdce(ilat,ilon) +
     $             quality(pix,lin)
            endif
  10     end do
      end do 

      return
      end


