      ! Write retrieved vertical structure to output file.

      subroutine output_structure(nwork, lat, lon, stdtime,
     $      pixel_status, rain_status, sfctype, sfcrain, convect,
     $      quality, lyrhgt, clw, rain, ciw, ice, Q1)

      implicit none

      include 'parameters.inc'


      integer     nwork
      real        lat(NPIX, MAX_SCANS)
      real        lon(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)
      integer*1   sfctype(NPIX, MAX_SCANS)
      real        sfcrain(NPIX, MAX_SCANS)
      real        convect(NPIX, MAX_SCANS)
      real        quality(NPIX, MAX_SCANS)
      real        lyrhgt(NPIX, MAX_SCANS, NLYR)
      real        clw (NPIX, MAX_SCANS, NLYR)
      real        rain(NPIX, MAX_SCANS, NLYR)
      real        ciw (NPIX, MAX_SCANS, NLYR)
      real        ice (NPIX, MAX_SCANS, NLYR)
      real        Q1 (NPIX, MAX_SCANS, NLYR)


      integer     lin, pix, lyr, i
      integer*1   rain_mask(NPIX, MAX_SCANS)
      integer     index
      integer     structure_flag(MAX_SCANS)

      
 
      !  Create the rainfall mask for output
      ! rain_mask -1  => No rain was retrieved
      ! rain_mask  0  => Rain was retrieved but it may be bad
      ! rain_mask +1  => Rain was retrieved but = 0 due to screen
      ! rain_mask +2  => A rainfall retrieval was performed
c
      do lin = 1, nwork
         do pix = 1, NPIX

            ! initialize all pixels to O.K.
            rain_mask(pix,lin) = 2

            ! For pixels w/o retrievals, rain value never changes
            ! from initialized -999.
            if ( sfcrain(pix,lin) .lt. 0.) rain_mask(pix,lin) = -1

            ! If an anomalous condition exists with rainfall,
            ! rain_status > 0 was assigned.  These pixels are now
            ! lumped into the rain_mask = 0 category.
            if (rain_status(pix,lin) .gt. 0) rain_mask(pix,lin) = 0

            ! Set the "screened" pixels to +1
            if (rain_status(pix,lin) .lt. 0) rain_mask(pix,lin) = 1
            
         end do
      end do 

      ! Determine if "structure" output can be reduced
      ! structure_flag -1  => Rain is not retrieved in scan line.
      ! structure_flag  0  => All rain was screened from line.  
      !                       There is no rainfall structure.
      ! structure_flag +1  => Rain structure is available.

      do lin = 1, nwork

         ! identify the first pixel in the scan line
         structure_flag(lin) = 1
         if (rain_mask(1,lin) .eq. -1 ) then
            structure_flag(lin) = -1  
         else if (rain_mask(1,lin) .eq.  1 ) then
            structure_flag(lin) =  0
         endif
   
         do pix = 2, NPIX

            ! Compare remaining pixels with first one 
            index = 1
            if (rain_mask(pix,lin) .eq. -1 ) then
               index = -1  
            else if (rain_mask(pix,lin) .eq.  1 ) then
               index =  0
            endif

            if ( index .ne. structure_flag(lin)) then
               structure_flag(lin) = +1
               goto 10
            end if
         end do
10    end do

      do lin = 1, nwork
         write(9) lin      ! DEBUG
         write(9) (lat(pix, lin), pix=1, NPIX)
         write(9) (lon(pix, lin), pix=1, NPIX)
         write(9) (stdtime(lin, i), i = 1, 6)
         write(9) (rain_status(pix, lin), pix=1, NPIX)
         write(9) (pixel_status(pix, lin), pix=1, NPIX)
         write(9) (sfctype(pix, lin), pix=1, NPIX)
         write(9) (sfcrain(pix, lin), pix=1, NPIX)
         write(9) (convect(pix, lin), pix=1, NPIX)
         write(9) (quality(pix, lin), pix=1, NPIX)
         write(9) (lyrhgt(1,lin,lyr), lyr = 1, NLYR)
         write(9) structure_flag(lin)
         if ( structure_flag(lin) .eq. 1 ) then
            write(9) ((clw(pix,lin,lyr), pix=1, NPIX), lyr=1, NLYR)
            write(9) ((rain(pix,lin,lyr), pix=1, NPIX), lyr=1, NLYR)
            write(9) ((ciw(pix,lin,lyr), pix=1, NPIX), lyr=1, NLYR)
            write(9) ((ice(pix,lin,lyr), pix=1, NPIX), lyr=1, NLYR)
            write(9) ((Q1(pix,lin,lyr), pix=1, NPIX), lyr=1, NLYR)
         else
            write(9) (clw(1,lin,lyr), lyr=1, NLYR)
            write(9) (rain(1,lin,lyr), lyr=1, NLYR)
            write(9) (ciw(1,lin,lyr), lyr=1, NLYR)
            write(9) (ice(1,lin,lyr), lyr=1, NLYR)
            write(9) (Q1(1,lin,lyr), lyr=1, NLYR)
         end if
      end do

      return
      end


